#! /usr/bin/perl -w
#@ _TITLE_
# ext_num_sort.pl - sort filenames based on numeric extension value.
#@ _TITLE_

# Purpose: to sort a set of update log files into proper order for
# pumping into mysql to re-execute the queries in the logs.

# Suppose you have this set of files:
#    update.998 update.999 update.1000
# ext_num_sort.pl will order them that way.  That is different than
# ls and sort, which would put them in this order:
#    update.1000 update.998 update.999

# Filenames to be sorted should be presented as input to the script,
# one name per line.  Example:
#    % ls update.[0-9]* | ext_num_sort.pl | mysql
# Output is the same set of files, sorted properly.

# It's assumed that the basename for all the files is the same.

#@ _MAIN_
use strict;

my @files = <>;			# read all input lines
@files = sort {			# sort them by numeric extension
			my $anum = $1 if $a =~ /\.(\d+)$/;
			my $bnum = $1 if $b =~ /\.(\d+)$/;
			$anum <=> $bnum;
		} @files;
print @files;			# print them
exit (0);
#@ _MAIN_
