#! /usr/bin/perl -w
#@ _SETUP_

use strict;
use DBI;

# parse connection parameters from command line if given

use Getopt::Long;
$Getopt::Long::ignorecase = 0; # options are case sensitive
$Getopt::Long::bundling = 1;   # -uname = -u name, not -u -n -a -m -e

# default parameters - all undefined initially
my ($host_name, $password, $port_num, $socket_name, $user_name);

GetOptions (
	# =i means an integer argument is required after option
	# =s or :s means string argument is required or optional after option
	"host|h=s"      => \$host_name,
	"password|p:s"  => \$password,
	"port|P=i"      => \$port_num,
	"socket|S=s"    => \$socket_name,
	"user|u=s"      => \$user_name
) or exit (1);

# solicit password if option was specified without option value
if (defined ($password) && !$password)
{
	# turn off echoing but don't interfere with STDIN
	open (TTY, "/dev/tty") or die "Cannot open terminal\n";
	system ("stty -echo < /dev/tty");
	print STDERR "Enter password: ";
	chomp ($password = <TTY>);
	system ("stty echo < /dev/tty");
	close (TTY);
	print STDERR "\n";
}

# construct data source
my $dsn = "DBI:mysql:sampdb";
$dsn .= ";host=$host_name" if $host_name;
$dsn .= ";port=$port_num" if $port_num;
$dsn .= ";mysql_socket=$socket_name" if $socket_name;
$dsn .= ";mysql_read_default_group=client";

# connect to server
my $dbh = DBI->connect ($dsn, $user_name, $password,
						{ RaiseError => 1, PrintError => 0, AutoCommit => 1 });
#@ _SETUP_

#@ _TEARDOWN_
$dbh->disconnect ();
exit (0);
#@ _TEARDOWN_
