<?php
#@ _OUTLINE_PART_1_
# score_entry.php - Score Entry script for grade-keeping project

require_once "DB.php";
require_once "sampdb.php";

# define action constants
define ("SHOW_INITIAL_PAGE", 0);
define ("SOLICIT_EVENT", 1);
define ("ADD_EVENT", 2);
define ("DISPLAY_SCORES", 3);
define ("ENTER_SCORES", 4);
#@ _OUTLINE_PART_1_

#@ _DISPLAY_CELL_
# Display a cell of an HTML table.  $tag is the tag name ("th" or "td"
# for a header or data cell), $value is the value to display, and
# $encode should be true or false, indicating whether or not to perform
# HTML-encoding of the value before displaying it.  $encode is optional,
# and is true by default.

function display_cell ($tag, $value, $encode = TRUE)
{
	if ($value == "")	# is the value empty or unset?
		$value = "&nbsp;";
	else if ($encode)	# perform HTML-encoding if requested
		$value = htmlspecialchars ($value);
	print ("<$tag>$value</$tag>\n");
}
#@ _DISPLAY_CELL_

# Display a list of the existing grade events.  User can select any
# of them to add or edit scores for that event.  There is also a "new
# event" link for creating a new event.

#@ _DISPLAY_EVENTS_
function display_events ($conn)
{
	print ("Select an event by clicking on its number, or select\n");
	print ("New Event to create a new grade event:<br /><br />\n");
	$stmt = "SELECT event_id, date, category
				FROM grade_event ORDER BY event_id";
	$result =& $conn->query ($stmt);
	if (DB::isError ($result))
		die ("Cannot execute statement");
	print ("<table border=\"1\">\n");

	# Print a row of table column headers

	print ("<tr>\n");
	display_cell ("th", "Event ID");
	display_cell ("th", "Date");
	display_cell ("th", "Category");
	print ("</tr>\n");

	# Present list of existing events.  Associate each event id with a
	# link that will show the scores for the event; use DB_FETCHMODE_ASSOC
	# to fetch each row so that its columns can be referred to by name.

	while ($row =& $result->fetchRow (DB_FETCHMODE_ASSOC))
	{
		print ("<tr>\n");
		$url = sprintf ("%s?action=%s&event_id=%s",
						script_name (),
						urlencode (DISPLAY_SCORES),
						urlencode ($row["event_id"]));
		display_cell ("td",
					"<a href=\"$url\">"
						. htmlspecialchars ($row["event_id"])
						. "</a>",
					FALSE);
		display_cell ("td", $row["date"]);
		display_cell ("td", $row["category"]);
		print ("</tr>\n");
	}
	$result->free ();

	# Add one more link for creating a new event

	print ("<tr align=\"center\">\n");
	$url = sprintf ("%s?action=%s",
					script_name (),
					urlencode (SOLICIT_EVENT));
	display_cell ("td colspan=\"3\"",
					"<a href=\"$url\">" . "Create New Event" . "</a>",
					FALSE);
	print ("</tr>\n");

	print ("</table>\n");
}
#@ _DISPLAY_EVENTS_

# present form to solicit information for new event

#@ _SOLICIT_EVENT_INFO_
function solicit_event_info ()
{
	printf ("<form method=\"post\" action=\"%s?action=%s\">\n",
				script_name (),
				urlencode (ADD_EVENT));
	print ("Enter information for new grade event:<br /><br />\n");
	print ("Date: ");
	print ("<input type=\"text\" name=\"date\" value=\"\" size=\"10\" />\n");
	print ("<br />\n");
	print ("Category: ");
	print ("<input type=\"radio\" name=\"category\" value=\"T\"");
	print (" checked=\"checked\" />Test\n");
	print ("<input type=\"radio\" name=\"category\" value=\"Q\" />Quiz\n");
	print ("<br /><br />\n");
	print ("<input type=\"submit\" name=\"button\" value=\"Submit\" />\n");
	print ("</form>\n");
}
#@ _SOLICIT_EVENT_INFO_

# add new event to event table

#@ _ADD_NEW_EVENT_
function add_new_event ($conn)
{
	$date = script_param ("date");	# get date and event category
	$category = script_param ("category");	# entered by user

	if (empty ($date))	# make sure a date was entered, and in ISO format
		die ("No date specified");
	if (!preg_match ('/^\d+\D\d+\D\d+$/', $date))
		die ("Please enter the date in ISO format (CCYY-MM-DD)");
	if ($category != "T" && $category != "Q")
		die ("Bad event category");

	$stmt = "INSERT INTO grade_event (date,category) VALUES(?,?)";
	if ($conn->query ($stmt, array ($date, $category)) != DB_OK)
		die ("Could not add event to database");
}
#@ _ADD_NEW_EVENT_

# Display all scores for a given event, sorted by student name.
# Names are displayed as static text, scores as editable fields.

#@ _DISPLAY_SCORES_
function display_scores ($conn)
{
	# Get event ID number, which must look like an integer
	$event_id = script_param ("event_id");
	if (!preg_match ('/^\d+$/', $event_id))
		die ("Bad event ID");

	# select scores for the given event
	$stmt = "
		SELECT
            student.student_id, student.name, grade_event.date,
			score.score AS score, grade_event.category
		FROM student, grade_event
			LEFT JOIN score ON student.student_id = score.student_id
                    AND grade_event.event_id = score.event_id
		WHERE grade_event.event_id = ?
		ORDER BY student.name";
	$result =& $conn->query ($stmt, array ($event_id));
	if (DB::isError ($result))
		die ("Cannot execute statement");
	if ($result->numRows () < 1)
		die ("No information was found for the selected event");

	printf ("<form method=\"post\" action=\"%s?action=%s&event_id=%s\">\n",
				script_name (),
				urlencode (ENTER_SCORES),
				urlencode ($event_id));

	# print scores as an HTML table

	$row_num = 0;
	while ($row =& $result->fetchRow (DB_FETCHMODE_ASSOC))
	{
		# print event info and table heading preceding the first row
		if ($row_num == 0)
		{
			printf ("Event ID: %s, Event date: %s, Event category: %s\n",
						htmlspecialchars ($event_id),
						htmlspecialchars ($row["date"]),
						htmlspecialchars ($row["category"]));
			print ("<br /><br />\n");
			print ("<table border=\"1\">\n");
			print ("<tr>\n");
			display_cell ("th", "Name");
			display_cell ("th", "Score");
			print "</tr>\n";
		}
		++$row_num;
		print ("<tr>\n");
		display_cell ("td", $row["name"]);
		$col_val = sprintf ("<input type=\"text\" name=\"score[%s]\"",
								htmlspecialchars ($row["student_id"]));
		$col_val .= sprintf (" value=\"%s\" size=\"5\" /><br />\n",
								htmlspecialchars ($row["score"]));
		display_cell ("td", $col_val, FALSE);
		print ("</tr>\n");
	}
	$result->free ();

	print ("</table>\n");
	print ("<br />\n");
	print ("<input type=\"submit\" name=\"button\" value=\"Submit\" />\n");
	print "</form>\n";
}
#@ _DISPLAY_SCORES_

#@ _ENTER_SCORES_
function enter_scores ($conn)
{
	# Get event ID number and array of scores for the event

	$event_id = script_param ("event_id");
#@ _GET_SCORE_PARAM_
	$score = script_param ("score");
#@ _GET_SCORE_PARAM_

	if (!preg_match ('/^\d+$/', $event_id))	# must look like integer
		die ("Bad event ID");

	# Prepare the statements that are executed repeatedly
	$sth_del = $conn->prepare ("DELETE FROM score
								WHERE event_id = ? AND student_id = ?");
	if (DB::isError ($sth_del))
		die ("Cannot prepare statement");
	$sth_repl = $conn->prepare ("REPLACE INTO score
									(event_id,student_id,score)
									VALUES(?,?,?)");
	if (DB::isError ($sth_repl))
		die ("Cannot prepare statement");

	# enter scores within a transaction
	if ($conn->autoCommit (FALSE) != DB_OK)
		die ("Cannot disable autocommit");

	$err = 0;
	$blank_count = 0;
	$nonblank_count = 0;
	reset ($score);
	while (list ($student_id, $new_score) = each ($score))
	{
		$new_score = trim ($new_score);
		if (empty ($new_score))
		{
			# if no score is provided for student in the form, delete any
			# score the student may have had in the database previously
			++$blank_count;
			$sth =& $del_sth;
			$params = array ($event_id, $student_id);
		}
		else if (preg_match ('/^\d+$/', $new_score)) # must look like integer
		{
			# if a score is provided, replace any score that
			# might already be present in the database
			++$nonblank_count;
			$sth =& $sth_repl;
			$params = array ($event_id, $student_id, $new_score);
		}
		else
		{
			print ("invalid score: $new_score");
			$err = 1;
			break;
		}
		if ($conn->execute ($sth, $params) != DB_OK)
		{
			print ("score entry failed, event_id $event_id,"
					. " student_id $student_id");
			$err = 1;
			break;
		}
	}

	# Commit or roll back the transaction, depending on whether
	# score entry succeeded or failed. Then re-enable autocommit.

	if ($err == 0)
	{
		if ($conn->commit () != DB_OK)
			die ("Cannot commit transaction");
	}
	else
	{
		if ($conn->rollback () != DB_OK)
			die ("Cannot roll back transaction");
	}
	$conn->autoCommit (TRUE);

	# Release the prepared statements
	$conn->freePrepared ($sth_del);
	$conn->freePrepared ($sth_repl);

	printf ("Number of scores entered: %d<br />\n", $nonblank_count);
	printf ("Number of scores missing: %d<br />\n", $blank_count);
	print ("<br />\n");
}
#@ _ENTER_SCORES_

#@ _OUTLINE_PART_2_
$title = "Grade-Keeping Project -- Score Entry";
html_begin ($title, $title);

$conn =& DB::connect (sampdb_dsn ());
if (DB::isError ($conn))
	die ("Cannot connect to server");

# determine what action to perform (the default if
# none is specified is to present the initial page)

#@ _GET_ACTION_PARAM_
$action = script_param ("action");
#@ _GET_ACTION_PARAM_
if (!isset ($action))
	$action = SHOW_INITIAL_PAGE;

switch ($action)
{
case SHOW_INITIAL_PAGE:		# present initial page
	display_events ($conn);
	break;
case SOLICIT_EVENT:			# ask for new event information
	solicit_event_info ();
	break;
case ADD_EVENT:				# add new event to database
	add_new_event ($conn);
	display_events ($conn);
	break;
case DISPLAY_SCORES:		# display scores for selected event
	display_scores ($conn);
	break;
case ENTER_SCORES:			# enter new or edited scores
	enter_scores ($conn);
	display_events ($conn);
	break;
default:
	die ("Unknown action code ($action)");
}

$conn->disconnect ();

html_end ();
#@ _OUTLINE_PART_2_
?>
