<?php
#@ _OUTLINE_PART_1_
# edit_member.php - Edit U.S. Historical League member entries via the Web

require_once "DB.php";
require_once "sampdb.php";

# define action constants
define ("SHOW_INITIAL_PAGE", 0);
define ("DISPLAY_ENTRY", 1);
define ("UPDATE_ENTRY", 2);
#@ _OUTLINE_PART_1_

# display a login form to solicit the member's ID and password

#@ _SOLICIT_MEMBER_ID_
function display_login_page ()
{
	printf ("<form method=\"post\" action=\"%s?action=%s\">\n",
				script_name (),
				urlencode (DISPLAY_ENTRY));
	print ("Enter your membership ID number and password,\n");
	print ("then select Submit.\n<br /><br />\n");
	print ("<table>\n");
	print ("<tr>");
	print ("<td>Member ID</td><td>");
	text_field ("member_id", "", 10);
	print ("</td></tr>");
	print ("<tr>");
	print ("<td>Password</td><td>");
	password_field ("password", "", 10);
	print ("</td></tr>");
	print ("</table>\n");
	submit_button ("button", "Submit");
	print "</form>\n";
}
#@ _SOLICIT_MEMBER_ID_

# Display a column of a member entry.  $label is the visible label
# displayed to the user.  $row is the array containing the member
# entry.  $col_name is the name of a column in the entry.  $editable
# should be true if the user is allowed to change the column value.
# (The value is displayed as non-editable text otherwise.) $editable
# is optional; if missing, it defaults to true.  Field names are
# constructed using row[col_name] format so that when the form is
# submitted, values can be accessed using a $row array rather than a
# bunch of individual variables.

#@ _DISPLAY_COLUMN_
function display_column ($label, $row, $col_name, $editable = TRUE)
{
	print ("<tr>\n");
	printf ("<td>%s</td>\n", htmlspecialchars ($label));
	print ("<td>");
	if ($editable)	# display as edit field
		text_field ("row[$col_name]", $row[$col_name], 80);
	else			# display as read-only text
		print (htmlspecialchars ($row[$col_name]));
	print ("</td>\n");
	print ("</tr>\n");
}
#@ _DISPLAY_COLUMN_

#@ _CHECK_PASS_
function check_pass ($conn, $id, $pass)
{
	$stmt = "SELECT password FROM member_pass WHERE member_id = ?";
	$result =& $conn->query ($stmt, array ($id));
	if (DB::isError ($result))
		die ("Error reading password table");
	if ($row =& $result->fetchRow(DB_FETCHMODE_ASSOC))
		return ($row["password"] == $pass); # TRUE if password matches
	return (FALSE);							# no record found
}
#@ _CHECK_PASS_

# Determine whether the client supplied the correct password for
# a member entry.  If so, display the entry for editing.

#@ _DISPLAY_ENTRY_
function display_entry ($conn)
{
	# Get script parameters; trim whitespace from the ID, but not
	# from the password, because the password must match exactly.

	$member_id = trim (script_param ("member_id"));
	$password = script_param ("password");

	if (empty ($member_id))
		die ("No member ID was specified");
	if (!preg_match ('/^\d+$/', $member_id))	# must look like integer
		die ("Invalid member ID was specified (must be an integer)");
	if (empty ($password))
		die ("No password was specified");
	if (check_pass ($conn, $member_id, $password))	# regular member
		$admin = FALSE;
	else if (check_pass ($conn, 0, $password))		# administrator
		$admin = TRUE;
	else
		die ("Invalid password");

	$stmt = "SELECT
					last_name, first_name, suffix, email, street, city,
					state, zip, phone, interests, member_id, expiration
				FROM member WHERE member_id = ?
				ORDER BY last_name";
	$result =& $conn->query ($stmt, array ($member_id));
	if (DB::isError ($result))
		die ("Cannot execute statement");
	if ($result->numRows () == 0)
		die ("No user with member_id = $member_id was found");
	if ($result->numRows () > 1)
		die ("More than one user with member_id = $member_id was found");

	printf ("<form method=\"post\" action=\"%s?action=%s\">\n",
				script_name (),
				urlencode (UPDATE_ENTRY));
	
	# Add member ID and password as hidden values so that next invocation
	# of script can tell which record the form corresponds to and so that
	# the user need not re-enter the password.

	hidden_field ("member_id", $member_id);
	hidden_field ("password", $password);

	# Read results of statement and format for editing

	$row =& $result->fetchRow(DB_FETCHMODE_ASSOC);

	print ("<table>\n");

	# Display member ID as static text

	display_column ("Member ID", $row, "member_id", FALSE);

	# $admin is true if the user provided the administrative password,
	# false otherwise. Administrative users can edit the expiration
	# date, regular users cannot.

	display_column ("Expiration", $row, "expiration", $admin);

	# Display other values as editable text

	display_column ("Last name", $row, "last_name");
	display_column ("First name", $row, "first_name");
	display_column ("Suffix", $row, "suffix");
	display_column ("Email", $row, "email");
	display_column ("Street", $row, "street");
	display_column ("City", $row, "city");
	display_column ("State", $row, "state");
	display_column ("Zip", $row, "zip");
	display_column ("Phone", $row, "phone");
	display_column ("Interests", $row, "interests");

	print ("</table>\n");

	submit_button ("button", "Submit");
	print "</form>\n";

}
#@ _DISPLAY_ENTRY_

#@ _UPDATE_ENTRY_
function update_entry ($conn)
{
	# Get script parameters; trim whitespace from the ID, but not
	# from the password, because the password must match exactly,
	# or from the row, because it is an array.

	$member_id = trim (script_param ("member_id"));
	$password = script_param ("password");
	$row = script_param ("row");

	$member_id = trim ($member_id);
	if (empty ($member_id))
		die ("No member ID was specified");
	if (!preg_match ('/^\d+$/', $member_id))	# must look like integer
		die ("Invalid member ID was specified (must be an integer)");
	if (!check_pass ($conn, $member_id, $password)
			&& !check_pass ($conn, 0, $password))
		die ("Invalid password");

	# Examine the metadata for the member table to determine whether
	# each column allows NULL values.

	$info =& $conn->tableInfo ("member");
	if (DB::isError ($info))
		die ("Cannot get member table metadata");
	$nullable = array ();
	for ($i = 0; $i < count ($info); $i++)
	{
		$col_info = $info[$i];
		$col_name = $col_info["name"];
		$col_flags = explode (" ", $col_info["flags"]);
		$nullable[$col_name] = TRUE;	# TRUE unless we find not_null
		while (list ($key, $val) = each ($col_flags))
		{
			if ($val == "not_null")
			{
				$nullable[$col_name] = FALSE;
				break;
			}
		}
	}

	# Iterate through each field in the form, using the values to
	# construct the UPDATE statement.

	$stmt = "UPDATE member ";
	$delim = "SET";
	reset ($row);
	while (list ($col_name, $val) = each ($row))
	{
		$stmt .= "$delim $col_name=";
		$delim = ",";
		# if a form value is empty, update the corresponding column value
		# with NULL if the column is nullable.  This prevents trying to
		# put an empty string into the expiration date column when it
		# should be NULL, for example.
		$val = trim ($val);
		if (empty ($val))
		{
			if ($nullable[$col_name])
				$stmt .= "NULL";	# enter NULL
			else
				$stmt .= "''";		# enter empty string
		}
		else
			$stmt .= $conn->quoteSmart ($val);
	}
	$stmt .= sprintf (" WHERE member_id = %s", $conn->quoteSmart ($member_id));
	$result =& $conn->query ($stmt);
	if (DB::isError ($result))
		print ("Member entry was not updated.\n");
	else
		print ("Member entry was updated successfully.\n");
	printf ("<br /><a href=\"%s\">Edit another member record</a>\n",
			script_name ());
}
#@ _UPDATE_ENTRY_

#@ _OUTLINE_PART_2_
$title = "U.S. Historical League -- Member Editing Form";
html_begin ($title, $title);

$conn =& DB::connect (sampdb_dsn ());
if (DB::isError ($conn))
	die ("Cannot connect to server");

# determine what action to perform (the default if
# none is specified is to present the initial page)

$action = script_param ("action");
if (!isset ($action))
	$action = SHOW_INITIAL_PAGE;

switch ($action)
{
case SHOW_INITIAL_PAGE:		# present initial page
	display_login_page ();
	break;
case DISPLAY_ENTRY:			# display entry for editing
	display_entry ($conn);
	break;
case UPDATE_ENTRY:			# store updated entry in database
	update_entry ($conn);
	break;
default:
	die ("Unknown action code ($action)");
}

$conn->disconnect ();

html_end ();
#@ _OUTLINE_PART_2_
?>
